package com.soywiz.korio.compression

import com.soywiz.korio.compression.deflate.*
import com.soywiz.korio.compression.lzma.*
import com.soywiz.korio.lang.*
import com.soywiz.korio.util.checksum.*
import com.soywiz.korio.util.encoding.*
import kotlin.test.*

class ZLibTest {
	val data = ByteArray(0x20_000).apply {
		for (n in 0 until this.size) {
			val m = (n * 77) and 0xFF
			val v = when {
				m < 16 -> 0
				m < 32 -> m
				else -> 1
			}
			this[n] = v.toByte()
		}
	}

	private val data1 = ("" +
			"eJztzydhGAAAALDd/v////////////////////////////////9no6BxkF+///z99x8A" +
			"EAgYBBQMHAISChoGFg4eAREJGQUVDR0DEwsbBxcPn4CQiJiElIycgpKKmoaWjp6BkYmZ" +
			"hZWNnYOTi5uHl49fQFBIWERUTFxCUkpaRlZOXkFRSVlFVU1dQ1NLW0dXT9/A0MjYxNTM" +
			"3MLSytrG1s7ewdHJ2cXVzd3D08vbx9fPPyAwKDgkNCw8IjIqOiY2Lj4hMSk5JTUtPSMz" +
			"KzsnNy+/oLCouKS0rLyisqq6prauvqGxqbmlta29o7Oru6e3r39gcGh4ZHRsfGJyanpm" +
			"dm5+YXFpeWV1bX1jc2t7Z3dv/+Dw6Pjk9Oz84vLq+ub27v7h8en55fXt/ePz5//z//n/" +
			"/H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f" +
			"/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z" +
			"//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+" +
			"P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//" +
			"5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n/" +
			"/H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f" +
			"/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z" +
			"//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+" +
			"P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//" +
			"5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n/" +
			"/H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f" +
			"/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z" +
			"//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+" +
			"P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//" +
			"5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n/" +
			"/H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f" +
			"/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z//n//H/+P/+f/8//5//z" +
			"//n//H/+P/+f/8//5//z//n//H/+3+P/Ba1OJPE=").fromBase64()


	@Test
	fun test() {
		assertEquals("hello", "78 DA CB 48 CD C9 C9 07 00 06 2C 02 15".unhexIgnoreSpaces.uncompress(ZLib).toString(UTF8))
		assertEquals("hello", "1F 8B 08 00 00 00 00 00 02 13 CB 48 CD C9 C9 07 00 86 A6 10 36 05 00 00 00".unhexIgnoreSpaces.uncompress(GZIP).toString(UTF8))
	}

	@Test
	fun zlibCompressDecompress() {
		val compressedData = data.compress(ZLib)
		val uncompressedData = compressedData.uncompress(ZLib)
		assertEquals(data.size, uncompressedData.size)
		for (n in 0 until data.size) assertEquals(data[n], uncompressedData[n])
	}

	@Test
	fun gzipCompressDecompress() {
		val compressedData = data.compress(GZIP)
		val uncompressedData = compressedData.uncompress(GZIP)
		assertEquals(data.size, uncompressedData.size)
		for (n in 0 until data.size) assertEquals(data[n], uncompressedData[n])
	}

	@Test
	fun deflateCompressDecompress() {
		val compressedData = data.compress(Deflate)
		val uncompressedData = compressedData.uncompress(Deflate)
		assertEquals(data.size, uncompressedData.size)
		for (n in 0 until data.size) assertEquals(data[n], uncompressedData[n])
	}

	@Test
	fun deflatePortableCompressDecompress() {
		val compressedData = data.compress(DeflatePortable)
		val uncompressedData = compressedData.uncompress(DeflatePortable)
		assertEquals(data.size, uncompressedData.size)
		for (n in 0 until data.size) assertEquals(data[n], uncompressedData[n])
	}

	@Test
	fun lzmaCompressDecompress() {
		val compressedData = data.compress(Lzma)
		val uncompressedData = compressedData.uncompress(Lzma)
		assertEquals(data.size, uncompressedData.size)
		for (n in 0 until data.size) assertEquals(data[n], uncompressedData[n])
	}

	@Test
	fun deflateData1() {
		data1.uncompress(ZLib { DeflatePortable(it) })
		"eJx9jq0NQCEMhG8YJMOwAgJdhWYA9kAyAIuQoBmkvNQ80Z+kP/l6uRy8urIysOVoprTaLlOlcXyzvE7qP3jBuo4XCDg/QAyZCQjI".fromBase64().uncompress(ZLib { DeflatePortable(it) })
		"eJxtjD0NgEAMhd8IBlDAigNMsDCxsSIIAyScikvOzAk4A02Tdujf8r2/FPl1Fctk8jufDzSGbGdJ17A7fbAZd6iowMMswM78tLjCy9+417UEYNsIeA==".fromBase64().uncompress(ZLib { DeflatePortable(it) })
		"eJx1jb8NQFAQxj+J2MAASo1EJGIEM1hAhV6iVEg0GgvozGEHG+jscDm5Q/L+FL/3+y73vjfAfVbciDXkqFR9lKretz3XWHbxxl44RSKb+9PWmFw/7+U+MvNlgtTa2D0s3jwyAsYZMgl65Qwx".fromBase64().uncompress(ZLib { DeflatePortable(it) })
		"H4sIAAAAAAAAA0tMTEwEAEXlmK0EAAAA".fromBase64().uncompress(GZIP { DeflatePortable })
		"H4sICOzl1k8AA2hvZ2UudHh0AMtIzcnJVyjPL8pJ4QIALTsIrwwAAAA=".fromBase64().uncompress(GZIP { DeflatePortable })
	}

	private val data2 = "eAHs1mN/J0kUxfET9MR2cmLbGtu2zeXYtidau7LGe9zq7F133Vrz/30a5/4+3Qcx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\n3+dK6V2FclBFdBW6Q0fUJ2kxMWfgh9BSenAtue6666677rrrrrvu//zVeBs=\n".fromBase64IgnoreSpaces()

	@Test
	fun deflateData2a() {
		assertEquals(345409182, CRC32.compute(data2.uncompress(ZLib { Deflate })))
	}

	@Test
	fun deflateData2b() {
		assertEquals(345409182, CRC32.compute(data2.uncompress(ZLib { DeflatePortable })))
	}

    @Test
    fun deflateDate3() {
        val compressedBase64 = "eNrs1zVQJEEUxvG3jrvNkl587u7u7mbZufvh7i65nbtbhFt+GeS4y7s3UzS11dfF\n" +
        "xtvTX9WvIORfTTesffnS9SCYk+yetIS4iRN8a32kg7SQV+QNGQHBzNIfGhou6jfx\n" +
        "+atZBA/AHpJJZoBc+0vukJfAzUz99AgI+9X5qwfARlLJbZB7meQ+GQc2E/bTQ2Dq\n" +
        "fkL9anZgM8fhg0fjHQJm7lf9alaib7/o8E+cPAIVlYXw+etr+PHrg0959+E5lFcW\n" +
        "Gg2C3Z5shun6T546CpXVxfD1+1v49eeTT/nw6SVUVhXpDcL+7u5Or/1BwS6IjA6E\n" +
        "WC0E4ty+JVYLNn52apju/NXoI4CTtBNkTp64gE1NLTg4OCgFveXY0bOsj+kgTlH/\n" +
        "qRMXsbWlDUdGRqSgtxw/ek7Yv23LAV07QWb3zsN48uQZPH/+/JQLFy7gjRs3MCEh\n" +
        "AbOysjAvLw/z8/N9QsLTJDx88DTrYzqIE0A9AMcIempsbMaBgQGp1NbU622846L+\n" +
        "luY2HB4elkp9fZOwny7BMYKeTp06Y1x4wi6+cekLCgp8VkJCst7GOw6gHoBnBJmy\n" +
        "0irs6+uTTnd3DyYnZfEX4DnfX15Wrf/XIJ2+vn5MSc7+r58uwTOCzOFDJ/DixYtT\n" +
        "Hj9+jIWFhVK4fOkG/wA8B5PPShaAx5YvXwITExPSQZyAWbNnArf5fP/KVcsAEaUD\n" +
        "gDB77myv/QGBLrBYLAZN0yAmJgasVqsU3O4YQb96AOLAY7FxsfqFkVJEZCRw0/h+\n" +
        "TYsDRJRSdHSU136Hw2Zc/rCwMOPy0/fSCAwKEPSrB8AFHrPZrDA+Pi4lq9UC3Jx8\n" +
        "v91ul/YBtFqtXvvZX8v4+Hj9q1T0s+X71QPAjX5RpOZtiCg1b9MvSnh4OLhc/9i7\n" +
        "Yx2EYSAGw4gMDH3/x80FpWJvWM/fL0VlZcDYvmv72Z/bHUQLwNMPwPf/2f99bXnw\n" +
        "IABzzrananIABwJwXdf+t2x5IALEOoC16kQAutp/AsABVHgEOBOAMQYB6AcHQACO\n" +
        "SsDWB7EOYJ/sDuB11gEECQDeMnCQA1jFASDZAegAOADkloBzEgAOALEOIH4MqAOA\n" +
        "RSAOgADAFIAAiACwCGQKwgEkQADq0AFYBBpjtD3gAEQAEQA6gMAIUCIAovcAbAIS\n" +
        "AATvAegAdACwCWgKoAOATUAloAiQAwHgAGpxAOAAlIA6ABgDigAiQAwEoNJvBlq5\n" +
        "i0AEgADUfXU7sAgAtwOLACIAbAISAAIQhSmAMaAOAPYAjAF1ALAHIAKIAEkQALcD\n" +
        "iwAwBciNAJ4KDC8GEQE4ABgDigAcAIwBTQEIQAheDur14CIAjAFDXw++DweASAGw\n" +
        "CLTCOwACIAKUB4KIABABEgVg3VcRAErAxA7gr+cBEIAvu2axJUUMRuH7PrwBhyXu\n" +
        "7rBiCUt0hdsad3d7B9zdocanXUtTQsjf1KImhFNsK5N7zjfSmq/kJi2mALQjjpOc\n" +
        "JbD+/lzlr14FaIvJ3wVgI5MoirQ8ARhjqjcBQ9mfsVDLFUAQMCT5/nQbbWf/JPml\n" +
        "8DcFUEImvh9o99o/jmO4ro8gCCGlJPvbtq1d+dE26NoOHEGev+f62hZArVpT+JsC\n" +
        "eIdMmo2OdieA7biIkxiO7ULKC9nf+jmg3ezfaneRxAlKo5Vc//7+YW2X/0+evFD4\n" +
        "mwK4h0xKozXYtqvJzPfn5GcBQ+AztFs2pNyT/d+8/oBqpabNG3/tdhe+F6DVauPH\n" +
        "dyvX/+2bT/jxw9Ju9ienmzfuKfxNAdwUDCOTwYESul2nsEv+KIrg9Q76Dp34vaV/\n" +
        "o96BlBFyV/k/uP8MpVIFcVw8dxozC8NeiZfLNTiuh3a7g48fPv+3/8nj52H97Ndm\n" +
        "5resfnL6l7/JpImTlwi4zMwZ8/iixcv4suWr+PIVxYLGTGMnB5UbOef5r127jm/c\n" +
        "tI1v37Gb79i5p0jQmGns5JDrP2vGkiUCLnP+3BX+/ftP7nkeZ4wVChozjZ0cVG7k\n" +
        "DJMxJXBQwMcJB43/mFAJHBTwcYLkTzHfA9gqOAT9czh1zcT4G39TAIlgg2Cx4Af0\n" +
        "CzktEaxPXdMYf+NvCiCb24IJgtXp3/2CCMVLJOhPHVanTreQn9/tzwEBAAAEACDw\n" +
        "f7Mf1INe/f0BAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\n" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\n" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAYAIAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\n" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\n" +
        "AAAAAAAAAIAbFlvzNI0=\n"
        val compressed = compressedBase64.fromBase64IgnoreSpaces()

        val uncompressed1 = compressed.uncompress(ZLib)
        val uncompressed2 = compressed.uncompress(ZLib { DeflatePortable(it) })

        assertTrue(uncompressed1.contentEquals(uncompressed2))
    }
}
